import { RabbitMQConfig, RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { RedisModule, RedisModuleOptions } from '@liaoliaots/nestjs-redis';
import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { CqrsModule } from '@nestjs/cqrs';
import { ConfigurationMessagerController } from './adapters/primaries/configuration-messager.controller';
import { RedisConfigurationRepository } from './adapters/secondaries/redis-configuration.repository';
import { DeleteConfigurationUseCase } from './domain/usecases/delete-configuration.usecase';
import { GetConfigurationUseCase } from './domain/usecases/get-configuration.usecase';
import { SetConfigurationUseCase } from './domain/usecases/set-configuration.usecase';

@Module({
  imports: [
    CqrsModule,
    RedisModule.forRootAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (
        configService: ConfigService,
      ): Promise<RedisModuleOptions> => ({
        config: {
          host: configService.get<string>('REDIS_HOST'),
          port: configService.get<number>('REDIS_PORT'),
          password: configService.get<string>('REDIS_PASSWORD'),
        },
      }),
    }),
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (
        configService: ConfigService,
      ): Promise<RabbitMQConfig> => ({
        exchanges: [
          {
            name: configService.get<string>('RMQ_EXCHANGE'),
            type: 'topic',
          },
        ],
        handlers: {
          setConfiguration: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: ['configuration.create', 'configuration.update'],
          },
          deleteConfiguration: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'configuration.delete',
          },
          propagateConfiguration: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'configuration.propagate',
          },
        },
        uri: configService.get<string>('RMQ_URI'),
        connectionInitOptions: { wait: false },
        enableControllerDiscovery: true,
      }),
    }),
  ],
  controllers: [ConfigurationMessagerController],
  providers: [
    GetConfigurationUseCase,
    SetConfigurationUseCase,
    DeleteConfigurationUseCase,
    RedisConfigurationRepository,
  ],
})
export class ConfigurationModule {}
